/* 
 * File:   Timer.cpp
 * Author: Will Matheson
 * 
 * Implementation for Timer (see header file for description)
 * 
 * Created on February 28, 2013, 5:21 PM
 */

#include "Timer.h"

//constructor (not defined)
Timer::Timer() {
}

//copy constructor (not defined)
Timer::Timer(const Timer& orig) {
}

//virtual destructor (not defined)
Timer::~Timer() {
}

//start the timer
void Timer::start() {
    //set up the "starting" timespec with the current monotonic "time"
    clock_gettime(CLOCK_MONOTONIC, &_startingTime);
}

//find how much time has passed (seconds)
unsigned long Timer::elapsedTimeSeconds() {
    //place to store the present monotonic "time"
    struct timespec currentTime;
    //place to store the elapsed seconds
    unsigned long seconds;    
    //get present monotonic "time"
    clock_gettime(CLOCK_MONOTONIC, &currentTime);
    //find the difference between the starting time and the present time
    seconds  = currentTime.tv_sec  - _startingTime.tv_sec;
    //return the difference in seconds
    return seconds;    
}

//find how much time has passed (milliseconds)
unsigned long Timer::elapsedTimeMilliseconds() {  
    struct timespec currentTime;
    unsigned long nanoseconds;    
    clock_gettime(CLOCK_MONOTONIC, &currentTime);
    nanoseconds = currentTime.tv_nsec - _startingTime.tv_nsec;
    //return the difference in milliseconds
    return (nanoseconds / 1000000);
}

//find how much time has passed (nanoseconds)
unsigned long Timer::elapsedTimeNanoseconds() {
    struct timespec currentTime;
    unsigned long nanoseconds;    
    clock_gettime(CLOCK_MONOTONIC, &currentTime);
    nanoseconds = currentTime.tv_nsec - _startingTime.tv_nsec;
    //return the difference in nanoseconds
    return nanoseconds;
}

//find if we've passed a certain time threshold (seconds)
bool Timer::isTimeoutSeconds(unsigned long threshold) {
    return threshold >= elapsedTimeSeconds();
}

//find if we've passed a certain time threshold (milliseconds)
bool Timer::isTimeoutMilliseconds(long double threshold) {
    return threshold >= elapsedTimeMilliseconds();
}

//find if we've passed a certain time threshold (nanoseconds)
bool Timer::isTimeoutNanoseconds(unsigned long threshold) {
    return threshold >= elapsedTimeNanoseconds();
}
