/* 
 * File:   main.cpp
 * Author: Will Matheson
 * Course: PROG 2400 (Data Structures), Section 700
 * Due:    April 18, 2013
 * 
 * PROG 2400 Assignment #4: Sorting
 * Part 1: Bubble, Insertion and Quicksorts
 * 
 * Performs bubble, insertion, and quicksorts on a list of 10,000 random
 * numbers. Generates files of unsorted and sorted numbers, and displays the
 * execution time of each sort for comparison purposes.
 * 
 * Created on April 9, 2013, 8:15 PM
 */

#include <cstdlib>
#include <iostream>
#include <fstream>
#include <time.h>
#include "Timer.h"

using namespace std;

//Partition: the key "sorting" part of quicksort
//everything less goes before the pivot, things equal or greater go ahead
int partition(int numbers[][4],
        int column, int left, int right, int pivotIndex){
    int pivotValue = numbers[pivotIndex][column];
    //move pivot to end: swap array[pivotIndex] and array[right]:
    numbers[pivotIndex][column] = numbers[right][column];
    numbers[right][column] = pivotValue;
    int storeIndex = left;
    for (int i=left; i<right; i++)
    {
        if (numbers[i][column] <= pivotValue)
        {
            //swap array[i] and array[storeIndex]
            int valueHolder = numbers[i][column];
            numbers[i][column] = numbers[storeIndex][column];
            numbers[storeIndex][column] = valueHolder;
            storeIndex ++;
        }
    }
    //move pivot to its final place
    //swap array[storeIndex] and array[right]
    int valueHolder = numbers[storeIndex][column];
    numbers[storeIndex][column] = numbers[right][column];
    numbers[right][column] = valueHolder;
    return storeIndex;
}

//the recursive part of the quicksort
//left: index of leftmost element in subarray
//right: index of rightmost element of subarray
void quickSortRecurs(int numbers[][4], int column, int left, int right){
    //if the sublist has two or more items
    if (left < right)
    {
        //choose any pivotIndex such that left <= pivotIndex <= right
        //can be left, can be right, can be anything in between
        int pivotIndex = right;//poor for already-nearly-sorted arrays
        
        //separate bigger and smaller items and get final position of pivot
        int pivotNewIndex = partition(numbers, column, left, right, pivotIndex);
        
        //recursively sort elements smaller than the pivot
        quickSortRecurs(numbers, column, left, pivotNewIndex - 1);
        
        //recursively sort elements at least as big as the pivot
        quickSortRecurs(numbers, column, pivotNewIndex + 1, right);
    }
}

//recursive quicksort - can go up to n^2 time but averages n log n
//http://en.wikipedia.org/wiki/Quicksort#In-place_version
unsigned long quickSort(int numbers[][4], int arraySize, int column){
    //start the timer
    Timer* myTimer = new Timer();
    myTimer->start();
    
    //DO QUICKSORT
    //indexes 0 through 9999 because the the first subarray is the whole thing
    quickSortRecurs(numbers, column, 0, (arraySize-1));        
    
    //return time elapsed
    unsigned long elapsedTime = myTimer->elapsedTimeMilliseconds();
    delete myTimer;
    return elapsedTime;
}

//standard insertion sort - runs in up to n^2 time but more adaptive than bubble
//http://en.wikipedia.org/wiki/Insertion_sort#Algorithm
unsigned long insertionSort(int numbers[][4], int arraySize, int column){
    //start the timer
    Timer* myTimer = new Timer();
    myTimer->start();
    
    //DO INSERTION SORT
    for (int i=1; i<arraySize; i++)
    {
        //pick up a value and make a conceptual hole where we did so
        int valuePickedUp = numbers[i][column];
        int holePosition = i;
        
        while ((holePosition > 0)
                && (valuePickedUp < numbers[holePosition-1][column]))
        {
            //shift larger value up
            numbers[holePosition][column] = numbers[holePosition-1][column];
            //move hole position down
            holePosition = holePosition - 1;
        }
        
        //insert!
        numbers[holePosition][column] = valuePickedUp;
    }
    
    //return time elapsed
    unsigned long elapsedTime = myTimer->elapsedTimeMilliseconds();
    delete myTimer;
    return elapsedTime;
}

//standard bubble sort - runs in up to n^2 time
unsigned long bubbleSort(int numbers[][4], int arraySize, int column){
    //start the timer
    Timer* myTimer = new Timer();
    myTimer->start();
    
    //DO BUBBLE SORT
    int checkTo = arraySize;//start by going all the way through
    do 
    {
        int progressPoint = 0;
        for (int i = 1; i < checkTo; i++)
        {
            //if the number behind is greater than number ahead
            if (numbers[i-1][column] > numbers[i][column])
            {
                int holder = numbers[i][column];
                numbers[i][column] = numbers[i-1][column];
                numbers[i-1][column] = holder;        
                progressPoint = i;
            }
        }
        
        checkTo = progressPoint; //now just go as far as you need to
        //the progressPoint was set as the top of the last swap from the
        //previous pass, therefore everything past this point is already sorted
        
    } while (checkTo != 0);//while there might be parts left that need sorting
    
    //return time elapsed
    unsigned long elapsedTime = myTimer->elapsedTimeMilliseconds();
    delete myTimer;
    return elapsedTime;
}

//generate random numbers and store them in all columns of the array
//http://cplusplus.com/reference/cstdlib/rand/
void generateNumbers(int numbers[][4], int arraySize, int rangeMax){
    //in the original development environment, 2,147,483,647 is the RAND_MAX
    //to view yours:
    //cout << RAND_MAX << endl;
    
    //initalize random seed
    srand (time(NULL));
    
    for (int i=0; i<arraySize; i++)
    {
        int number = rand() % (rangeMax + 1); //'between' 0 & 100,000 inclusive
        for (int j=0; j<4; j++)
        {
            numbers[i][j] = number;
        }
    }
}

//display all of the numbers
void viewNumbers(int numbers[][4], int arraySize)
{
    for (int i=0; i<arraySize; i++)
    {
        for (int j=0; j<4; j++)
        {
            cout << numbers[i][j];
            if (j<3)
            {
                cout << ",";            
            }
        }
        cout << endl;
        
        if (((i % 1000) == 0) && i != 0)
        {
            cout << "Press Return to continue: ";
            cin.get();            
            cout << endl;
        }
    }
}

//save one column of numbers to a file
void saveToFile(int numbers[][4], int arraySize,
        int arrayChoice, string outFileName){
    ofstream outStream(outFileName.c_str());//declare and open stream
    if(outStream.fail())
    {
	cerr << "Couldn't open output stream." 
                << " Do you have write privileges here? Is the disk full?" 
                << endl;
    }
    else
    {
        for (int i=0; i<arraySize; i++)
        {
            outStream << "Line " << (i + 1) << "," << numbers[i][arrayChoice];
            if (i < (arraySize-1))
            {
                outStream << endl;            
            }
        }
    }
    outStream.close();//close the stream
}

//entry point of program
int main(int argc, char** argv) {
    //we're going to generate and sort 10,000 random numbers btw. 0 and 100,000
    const int arraySize = 10000;
    const int rangeMax = 100000;
    //number of sorts we're doing
    const int numSorts = 3;
    //the array will hold the original unsorted numbers plus results of sorts
    //static int numbers[numSorts + 1][arraySize];
    static int numbers[arraySize][numSorts + 1];
    
    //this array will hold the elapsed times for each sort
    unsigned long timings[numSorts];
    //names of each sort
    string sortNames[] = {"Bubble Sort", "Insertion Sort", "QuickSort"};
    
    //generate the random numbers
    generateNumbers(numbers, arraySize, rangeMax);
    
    //perform sorts, saving their times
    //BUBBLE SORT
    timings[0] = bubbleSort(numbers, arraySize, 1);
    
    //INSERTION SORT
    timings[1] = insertionSort(numbers, arraySize, 2);
    
    //QUICKSORT
    timings[2] = quickSort(numbers, arraySize, 3);
    
    //view the sort results
    viewNumbers(numbers, arraySize);
    
    //display sort times
    cout << "ELAPSED TIMES (ms)" << endl;
    for (int i=0; i<numSorts; i++)
    {
        cout << sortNames[i] << ": " << timings[i] << endl;
    }
    
    //save unsorted numbers to file
    saveToFile(numbers, arraySize, 0, "usortnum.txt");
    
    //save sorted numbers to file (but it doesn't keep original line numbers)
    saveToFile(numbers, arraySize, 1, "sortnum.txt");

    return 0;
}
